export function buildTrieTree(keywords) {
    var rootNode = {};
    if (!keywords.length) {
        return rootNode;
    }
    var curNode = rootNode;
    // 遍历keywords数组，对每个keyword递归构造字典树
    for (var i = 0; i < keywords.length; i++) {
        var word = keywords[i];
        for (var j = 0, len = word.length; j < len; j++) {
            var char = word[j];
            if (!curNode.hasOwnProperty(char)) {
                curNode[char] = {};
            }
            curNode = curNode[char];
        }
        curNode.isLeaf = true;
        // 恢复curNode为rootNode(根节点)，准备开始扫描下一个keyword
        curNode = rootNode;
    }
    return rootNode;
}
export default function sliceTextByKeywords(_a) {
    var text = _a.text, keywords = _a.keywords, trieTreeRoot = _a.trieTreeRoot, _b = _a.enableLazyMatch, enableLazyMatch = _b === void 0 ? false : _b;
    if (!text) {
        return [];
    }
    if (!keywords || !keywords.length) {
        return [{
                type: 'normal',
                text: text,
                start: 0,
                end: text.length
            }];
    }
    var textTiles = [];
    var textLen = text.length;
    // 每次匹配起始位置
    var start = 0;
    // 上一次匹配终止下标，匹配到的字符串不包含text[lastMatchEnd]
    var lastMatchEnd = 0;
    var rootNode = trieTreeRoot ? trieTreeRoot : buildTrieTree(keywords);
    while (start < textLen) {
        var end = start;
        var curNode = rootNode;
        // 是否命中keyword字符串
        var matched = false;
        // 匹配到的keyword
        var matchecEndPos = end;
        while (end < textLen) {
            var char = text[end];
            if (curNode.hasOwnProperty(char)) {
                end++;
                curNode = curNode[char];
                if (curNode.isLeaf) {
                    matchecEndPos = end;
                    matched = true;
                }
                continue;
            }
            // 进行模糊匹配
            if (enableLazyMatch) {
                var upperChar = char.toLowerCase();
                var lowerChar = char.toLowerCase();
                if (curNode.hasOwnProperty(upperChar) || curNode.hasOwnProperty(lowerChar)) {
                    end++;
                    curNode = (curNode[upperChar] || curNode[lowerChar]);
                    if (curNode.isLeaf) {
                        matchecEndPos = end;
                        matched = true;
                    }
                    continue;
                }
            }
            break;
        }
        if (matched) {
            // 如果上次匹配结束时的下标位置小于本次匹配开始位置，说明中间有一段normal文本需要截取存储
            if (lastMatchEnd < start) {
                textTiles.push({
                    type: 'normal',
                    text: text.substring(lastMatchEnd, start),
                    start: lastMatchEnd,
                    end: start
                });
            }
            lastMatchEnd = matchecEndPos;
            textTiles.push({
                type: 'highlight',
                text: text.substring(start, matchecEndPos),
                start: start,
                end: matchecEndPos
            });
            start = matchecEndPos;
        }
        else {
            start++;
        }
    }
    // 最后一次匹配结束位置小于textLen表明还有一段normal文本没有截取
    if (lastMatchEnd < textLen) {
        textTiles.push({
            type: 'normal',
            text: text.substring(lastMatchEnd, textLen),
            start: lastMatchEnd,
            end: textLen
        });
    }
    return textTiles;
}
